/*
 * @(#)AbstractDocumentView.java  1.1.1  2006-04-11
 *
 * Copyright (c) 1996-2006 by the original authors of JHotDraw
 * and all its contributors ("JHotDraw.org")
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * JHotDraw.org ("Confidential Information"). You shall not disclose
 * such Confidential Information and shall use it only in accordance
 * with the terms of the license agreement you entered into with
 * JHotDraw.org.
 */

package org.jhotdraw.application;

import application.ApplicationContext;
import org.jhotdraw.application.action.SelectAllAction;
import java.io.*;
import java.util.*;
import org.jhotdraw.util.*;
import javax.swing.*;
import java.util.concurrent.*;
import java.util.prefs.*;
/**
 * AbstractDocumentView.
 * 
 * 
 * 
 * @author Werner Randelshofer
 * @version 1.1.1 2006-04-11 Fixed documentView file preferences.
 * <br>1.1 2006-02-16 Support for preferences added.
 * <br>1.0 January 3, 2006 Created.
 */
public abstract class AbstractDocumentView extends JPanel  implements DocumentView {
    private DocumentOrientedApplication application;
    /**
     * The file chooser used for saving the documentView.
     * Has a null value, if the file chooser has not been used yet.
     */
    protected JFileChooser saveChooser;
    /**
     * The file chooser used for opening the documentView.
     * Has a null value, if the file chooser has not been used yet.
     */
    protected JFileChooser openChooser;
    /**
     * The documentView file. 
     * Has a null value, if the documentView has not been loaded from a file
     * or has not been saved yet.
     */
    protected File file;
    /**
     * The executor used to perform background tasks for the DocumentView in a
     * controlled manner. This executor ensures that all background tasks
     * are executed sequentually.
     */
    protected Executor executor;
    /**
     * Hash map for storing documentView actions.
     */
    private HashMap actions;
    /**
     * This is set to true, if the documentView has unsaved changes.
     */
    private boolean hasUnsavedChanges;
    /**
     * The preferences of the documentView.
     */
    private Preferences prefs;
    
    /**
     * Creates a new instance.
     */
    public AbstractDocumentView() {
    }
    
    public void init() {
        prefs = Preferences.userNodeForPackage(getClass());
        initComponents();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        
        setLayout(new java.awt.BorderLayout());
        
    }//GEN-END:initComponents
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
    
    public DocumentOrientedApplication getApplication() {
        return (DocumentOrientedApplication) ApplicationContext.getInstance().getApplication();
    }
    
    public JComponent getComponent() {
        return this;
    }
    
    public File getFile() {
        return file;
    }
    
    public void setFile(File newValue) {
        File oldValue = file;
        file = newValue;
        if (prefs != null && newValue != null) {
            prefs.put("projectFile", newValue.getPath());
        }
        firePropertyChange("file", oldValue, newValue);
    }
    
    /**
     * Gets the open file chooser for the documentView.
     */
    public JFileChooser getOpenChooser() {
        if (openChooser == null) {
            openChooser = createOpenChooser();
        }
        return openChooser;
    }
    
    protected JFileChooser createOpenChooser() {
        JFileChooser c = new JFileChooser();
        if (prefs != null) {
            c.setSelectedFile(new File(prefs.get("projectFile", System.getProperty("user.home"))));
        }
        return c;
    }
    
    /**
     * Gets the save file chooser for the documentView.
     */
    public JFileChooser getSaveChooser() {
        if (saveChooser == null) {
            saveChooser = createSaveChooser();
        }
        return saveChooser;
    }
    
    protected JFileChooser createSaveChooser() {
        JFileChooser c = new JFileChooser();
        if (prefs != null) {
            c.setCurrentDirectory(new File(prefs.get("projectFile", System.getProperty("user.home"))));
        }
        return c;
    }
    /**
     * Returns true, if the documentView has unsaved changes.
     * This is a bound property.
     */
    public boolean isModified() {
        return hasUnsavedChanges;
    }
    /**
     * Gets rid of all the resources of the documentView.
     * No other methods should be invoked on the documentView afterwards.
     */
    public void destroy() {
        
    }
    /**
     * Returns the action with the specified id.
     */
    public Action getAction(String id) {
        return (actions == null) ? null : (Action) actions.get(id);
    }
    
    /**
     * Puts an action with the specified id.
     */
    public void putAction(String id, Action action) {
        if (actions == null) {
            actions = new HashMap();
        }
        if (action == null) {
            actions.remove(id);
        } else {
            actions.put(id, action);
        }
    }
    /**
     * Executes the specified runnable on the worker thread of the documentView.
     * Execution is perfomred sequentially in the same sequence as the
     * runnables have been passed to this method.
     */
    public void execute(Runnable worker) {
        if (executor == null) {
            executor = Executors.newSingleThreadExecutor();
        }
        executor.execute(worker);
    }
    
    public void setName(String n) {
        super.setName(n);
        if (n != null && n.startsWith("null")) {
            new Throwable().printStackTrace();
        }
    }
    
    public void setModified(boolean newValue) {
        boolean oldValue = hasUnsavedChanges;
        hasUnsavedChanges = newValue;
        firePropertyChange("modified", oldValue, newValue);
    }
    public void start() {
    }
    
    public void stop() {
    }
    
}
